@@ -29,9 +29,9 @@ import encodings.idna  # NOQA
 
 
 # used to get version
-from .conda_interface import cc
+from .conda_interface import pkgs_dirs
 from .conda_interface import envs_dirs, env_path_backup_var_exists, root_dir
-from .conda_interface import plan
+from .conda_interface import display_actions, execute_actions, execute_plan, install_actions
 from .conda_interface import get_index
 from .conda_interface import PY3
 from .conda_interface import package_cache
@@ -56,10 +56,9 @@ from conda_build.render import (parse_or_try_download, output_yaml, bldpkg_path,
 import conda_build.os_utils.external as external
 from conda_build.post import (post_process, post_build,
                               fix_permissions, get_build_metadata)
-from conda_build.metadata import build_string_from_metadata
 from conda_build.index import update_index
-from conda_build.create_test import (create_files, create_shell_files,
-                                     create_py_files, create_pl_files)
+from conda_build.create_test import (create_files, create_shell_files, create_r_files,
+                                     create_py_files, create_pl_files, create_lua_files)
 from conda_build.exceptions import indent
 from conda_build.features import feature_list
 
@@ -87,6 +86,7 @@ def prefix_files(prefix):
             path = join(root, dn)
             if islink(path):
                 res.add(path[len(prefix) + 1:])
+    res = set(utils.expand_globs(res, prefix))
     return res
 
 
@@ -188,8 +188,8 @@ def get_run_dists(m, config):
 
 
 def copy_recipe(m, config):
+    recipe_dir = join(config.info_dir, 'recipe')
     if config.include_recipe and m.include_recipe():
-        recipe_dir = join(config.info_dir, 'recipe')
         try:
             os.makedirs(recipe_dir)
         except:
@@ -197,8 +197,6 @@ def copy_recipe(m, config):
 
         if os.path.isdir(m.path):
             for fn in os.listdir(m.path):
-                if fn.startswith('.'):
-                    continue
                 src_path = join(m.path, fn)
                 dst_path = join(recipe_dir, fn)
                 utils.copy_into(src_path, dst_path, timeout=config.timeout, locking=config.locking)
@@ -212,6 +210,7 @@ def copy_recipe(m, config):
         rendered_metadata = copy.deepcopy(m)
         # fill in build versions used
         build_deps = []
+        # TODO: may be unnecessary after build_customization branch merge
         # we only care if we actually have build deps.  Otherwise, the environment will not be
         #    valid for inspection.
         if m.meta.get('requirements') and m.meta['requirements'].get('build'):
@@ -664,8 +663,8 @@ def create_env(prefix, specs, config, clear_cache=True, retry=0):
                 locks = []
                 try:
                     if config.locking:
-                        cc.pkgs_dirs = cc.pkgs_dirs[:1]
-                        locked_folders = cc.pkgs_dirs + list(config.bldpkgs_dirs)
+                        _pkgs_dirs = pkgs_dirs[:1]
+                        locked_folders = _pkgs_dirs + list(config.bldpkgs_dirs)
                         for folder in locked_folders:
                             if not os.path.isdir(folder):
                                 os.makedirs(folder)
@@ -679,15 +678,15 @@ def create_env(prefix, specs, config, clear_cache=True, retry=0):
 
                     with utils.try_acquire_locks(locks, timeout=config.timeout):
                         index = get_build_index(config=config, clear_cache=True)
-                        actions = plan.install_actions(prefix, index, specs)
+                        actions = install_actions(prefix, index, specs)
                         if config.disable_pip:
                             actions['LINK'] = [spec for spec in actions['LINK'] if not spec.startswith('pip-')]  # noqa
                             actions['LINK'] = [spec for spec in actions['LINK'] if not spec.startswith('setuptools-')]  # noqa
-                        plan.display_actions(actions, index)
+                        display_actions(actions, index)
                         if utils.on_win:
                             for k, v in os.environ.items():
                                 os.environ[k] = str(v)
-                        plan.execute_actions(actions, index, verbose=config.debug)
+                        execute_actions(actions, index, verbose=config.debug)
                         warn_on_old_conda_build(index=index)
                 except (SystemExit, PaddingError, LinkError, CondaError) as exc:
                     if (("too short in" in str(exc) or
@@ -717,7 +716,8 @@ def create_env(prefix, specs, config, clear_cache=True, retry=0):
                     #    The reason why we treat the "minimum conda version" text this way is that
                     #    it occurs with info/files is missing.  That's also a symptom of these
                     #    weird I/O issues that happen with parallel conda-build jobs.
-                    elif 'lock' in str(exc) or 'requires a minimum conda version' in str(exc):
+                    elif ('lock' in str(exc) or 'requires a minimum conda version' in str(exc) or
+                          'Cannot link a source that does not exist' in str(exc)):
                         if retry < config.max_env_retry:
                             log.warn("failed to create env, retrying.  exception was: %s", str(exc))
                             create_env(prefix, specs, config=config,
@@ -800,6 +800,7 @@ to get the latest version.
 
 
 def filter_files(files_list, prefix, filter_patterns=('(.*[\\\\/])?\.git[\\\\/].*',
+                                                      'conda-meta.*',
                                                       '(.*)?\.DS_Store.*')):
     """Remove things like .git from the list of files to be copied"""
     for pattern in filter_patterns:
@@ -809,30 +810,91 @@ def filter_files(files_list, prefix, filter_patterns=('(.*[\\\\/])?\.git[\\\\/].
             if not os.path.isdir(os.path.join(prefix, f))]
 
 
+def post_process_files(m, initial_prefix_files):
+    get_build_metadata(m, config=m.config)
+    create_post_scripts(m, config=m.config)
+
+    if not is_noarch_python(m):
+        utils.create_entry_points(m.get_value('build/entry_points'), config=m.config)
+    current_prefix_files = prefix_files(prefix=m.config.build_prefix)
+
+    post_process(sorted(current_prefix_files - initial_prefix_files),
+                    prefix=m.config.build_prefix,
+                    config=m.config,
+                    preserve_egg_dir=bool(m.get_value('build/preserve_egg_dir')),
+                    noarch=m.get_value('build/noarch'),
+                    skip_compile_pyc=m.get_value('build/skip_compile_pyc'))
+
+    # The post processing may have deleted some files (like easy-install.pth)
+    current_prefix_files = prefix_files(prefix=m.config.build_prefix)
+    new_files = sorted(current_prefix_files - initial_prefix_files)
+    new_files = filter_files(new_files, prefix=m.config.build_prefix)
+    if any(m.config.meta_dir in join(m.config.build_prefix, f) for f in new_files):
+        meta_files = (tuple(f for f in new_files if m.config.meta_dir in
+                join(m.config.build_prefix, f)),)
+        sys.exit(indent("""Error: Untracked file(s) %s found in conda-meta directory.
+This error usually comes from using conda in the build script.  Avoid doing this, as it
+can lead to packages that include their dependencies.""" % meta_files))
+    post_build(m, new_files, prefix=m.config.build_prefix, build_python=m.config.build_python,
+               croot=m.config.croot)
+
+    entry_point_script_names = get_entry_point_script_names(m.get_value('build/entry_points'))
+    if is_noarch_python(m):
+        pkg_files = [fi for fi in new_files if fi not in entry_point_script_names]
+    else:
+        pkg_files = new_files
+
+    # the legacy noarch
+    if m.get_value('build/noarch_python'):
+        noarch_python.transform(m, new_files, m.config.build_prefix)
+    # new way: build/noarch: python
+    elif is_noarch_python(m):
+        noarch_python.populate_files(m, pkg_files, m.config.build_prefix, entry_point_script_names)
+
+    current_prefix_files = prefix_files(prefix=m.config.build_prefix)
+    new_files = current_prefix_files - initial_prefix_files
+    fix_permissions(new_files, m.config.build_prefix)
+
+    return new_files
+
+
 def bundle_conda(output, metadata, config, env, **kw):
+    log = logging.getLogger(__name__)
+    log.info('Packaging %s', metadata.dist())
     files = output.get('files', [])
     if not files and output.get('script'):
         interpreter = output.get('script_interpreter')
         if not interpreter:
             interpreter = guess_interpreter(output['script'])
-        initial_files_snapshot = prefix_files(config.build_prefix)
+        initial_files = prefix_files(config.build_prefix)
         utils.check_call_env(interpreter.split(' ') +
                     [os.path.join(metadata.path, output['script'])],
                     cwd=config.build_prefix, env=env)
-        files = prefix_files(config.build_prefix) - initial_files_snapshot
-    tmp_metadata = copy.deepcopy(metadata)
-    tmp_metadata.meta['package']['name'] = output['name']
-    tmp_metadata.meta['requirements'] = {'run': output.get('requirements', [])}
+    else:
+        # we exclude the list of files that we want to keep, so post-process picks them up as "new"
+        keep_files = set(utils.expand_globs(files, config.build_prefix))
+        pfx_files = set(prefix_files(config.build_prefix))
+        initial_files = pfx_files - keep_files
+
+    files = post_process_files(metadata, initial_files)
 
     output_filename = ('-'.join([output['name'], metadata.version(),
-                                 build_string_from_metadata(tmp_metadata)]) + '.tar.bz2')
-    files = list(set(utils.expand_globs(files, config.build_prefix)))
+                                 metadata.build_id()]) + '.tar.bz2')
+    # first filter is so that info_files does not pick up ignored files
+    files = filter_files(files, prefix=config.build_prefix)
+    create_info_files(metadata, files, config=config, prefix=config.build_prefix)
+    for ext in ('.py', '.r', '.pl', '.lua', '.sh'):
+        test_dest_path = os.path.join(config.info_dir, 'recipe', 'run_test' + ext)
+        script = output.get('test', {}).get('script')
+        if script and script.endswith(ext):
+            utils.copy_into(os.path.join(metadata.path, output['test']['script']),
+                            test_dest_path, config.timeout, locking=config.locking)
+        elif os.path.isfile(test_dest_path) and metadata.meta.get('extra', {}).get('parent_recipe'):
+            # the test belongs to the parent recipe.  Don't include it in subpackages.
+            utils.rm_rf(test_dest_path)
+    # here we add the info files into the prefix, so we want to re-collect the files list
+    files = set(prefix_files(config.build_prefix)) - initial_files
     files = filter_files(files, prefix=config.build_prefix)
-    info_files = create_info_files(tmp_metadata, files, config=config, prefix=config.build_prefix)
-    info_files = filter_files(info_files, prefix=config.build_prefix)
-    for f in info_files:
-        if f not in files:
-            files.append(f)
 
     # lock the output directory while we build this file
     # create the tarball in a temporary directory to minimize lock time
@@ -865,9 +927,9 @@ def bundle_conda(output, metadata, config, env, **kw):
             verifier.verify_package(ignore_scripts=ignore_scripts, run_scripts=run_scripts,
                                     path_to_package=tmp_path)
         if config.output_folder:
-            output_folder = os.path.join(config.output_folder, config.subdir)
+            output_folder = os.path.join(config.output_folder, metadata.config.subdir)
         else:
-            output_folder = config.bldpkgs_dir
+            output_folder = metadata.config.bldpkgs_dir
         final_output = os.path.join(output_folder, output_filename)
         if os.path.isfile(final_output):
             os.remove(final_output)
@@ -981,12 +1043,14 @@ def build(m, config, post=None, need_source_download=True, need_reparse_in_env=F
                                                                 no_download_source=False,
                                                                 force_download=True,
                                                                 config=config)
+            m.final = True
             assert not need_source_download, "Source download failed.  Please investigate."
             if m.uses_jinja:
                 print("BUILD START (revised):", m.dist())
 
         if need_reparse_in_env:
             reparse(m, config=config)
+            m.final = True
             print("BUILD START (revised):", m.dist())
 
         print("Package:", m.dist())
@@ -1065,77 +1129,15 @@ def build(m, config, post=None, need_source_download=True, need_reparse_in_env=F
                         utils.check_call_env(cmd, env=env, cwd=src_dir)
 
     if post in [True, None]:
-        if post:
-            with open(join(config.croot, 'prefix_files.txt'), 'r') as f:
-                files1 = set(f.read().splitlines())
+        with open(join(m.config.croot, 'prefix_files.txt'), 'r') as f:
+            initial_files = set(f.read().splitlines())
 
-        get_build_metadata(m, config=config)
-        create_post_scripts(m, config=config)
-
-        if not is_noarch_python(m):
-            utils.create_entry_points(m.get_value('build/entry_points'), config=config)
-        files2 = prefix_files(prefix=config.build_prefix)
-
-        post_process(sorted(files2 - files1),
-                     prefix=config.build_prefix,
-                     config=config,
-                     preserve_egg_dir=bool(m.get_value('build/preserve_egg_dir')),
-                     noarch=m.get_value('build/noarch'),
-                     skip_compile_pyc=m.get_value('build/skip_compile_pyc'))
-
-        # The post processing may have deleted some files (like easy-install.pth)
-        files2 = prefix_files(prefix=config.build_prefix)
-        if any(config.meta_dir in join(config.build_prefix, f) for f in files2 - files1):
-            meta_files = (tuple(f for f in files2 - files1 if config.meta_dir in
-                    join(config.build_prefix, f)),)
-            sys.exit(indent("""Error: Untracked file(s) %s found in conda-meta directory.
-This error usually comes from using conda in the build script.  Avoid doing this, as it
-can lead to packages that include their dependencies.""" % meta_files))
-        post_build(m, sorted(files2 - files1),
-                    prefix=config.build_prefix,
-                    build_python=config.build_python,
-                    croot=config.croot)
-
-        entry_point_script_names = get_entry_point_script_names(m.get_value('build/entry_points'))
-        if is_noarch_python(m):
-            pkg_files = [fi for fi in sorted(files2 - files1) if fi not in entry_point_script_names]
-        else:
-            pkg_files = sorted(files2 - files1)
-
-        # the legacy noarch
-        if m.get_value('build/noarch_python'):
-            noarch_python.transform(m, sorted(files2 - files1), config.build_prefix)
-        # new way: build/noarch: python
-        elif is_noarch_python(m):
-            noarch_python.populate_files(
-                m, pkg_files, config.build_prefix, entry_point_script_names)
-
-        files3 = prefix_files(prefix=config.build_prefix)
-        fix_permissions(files3 - files1, config.build_prefix)
-
-        outputs = m.get_section('outputs')
-        # this is the old, default behavior: conda package, with difference between start
-        #    set of files and end set of files
-        requirements = m.get_value('requirements/run')
-        if not outputs:
-            outputs = [{'name': m.name(),
-                        'files': files3 - files1,
-                        'requirements': requirements}]
-        else:
-            made_meta = False
-            for out in outputs:
-                if out.get('name') in requirements:
-                    requirements.extend(out.get('requirements', []))
-                    made_meta = True
-            else:
-                # make a metapackage for the top-level package, but only if a matching output name
-                #    is not explicitly provided
-                if made_meta and not any(m.name() in out.get('name', '') for out in outputs):
-                    outputs.append({'name': m.name(), 'requirements': requirements})
+        files = prefix_files(prefix=m.config.build_prefix) - initial_files
+        outputs = m.get_output_metadata_set(files=files)
 
         output_folders = set()
-        for output in outputs:
-            built_package = bundlers[output.get('type', 'conda')](output, m, config, env)
+        for (output_dict, m) in outputs:
+            built_package = bundlers[output_dict.get('type', 'conda')](output_dict, m, config, env)
             built_packages.append(built_package)
             output_folders.add(os.path.dirname(built_package))
 
@@ -1167,21 +1169,21 @@ def guess_interpreter(script_filename):
 
 
 def clean_pkg_cache(dist, config):
-    pkgs_dirs = cc.pkgs_dirs[:1]
+    _pkgs_dirs = pkgs_dirs[:1]
     locks = []
     if config.locking:
-        locks = [utils.get_lock(folder, timeout=config.timeout) for folder in pkgs_dirs]
+        locks = [utils.get_lock(folder, timeout=config.timeout) for folder in _pkgs_dirs]
     with utils.try_acquire_locks(locks, timeout=config.timeout):
         rmplan = [
             'RM_EXTRACTED {0} local::{0}'.format(dist),
             'RM_FETCHED {0} local::{0}'.format(dist),
         ]
-        plan.execute_plan(rmplan)
+        execute_plan(rmplan)
 
         # Conda does not seem to do a complete cleanup sometimes.  This is supplemental.
         #   Conda's cleanup is still necessary - it keeps track of its own in-memory
         #   list of downloaded things.
-        for folder in cc.pkgs_dirs:
+        for folder in _pkgs_dirs:
             try:
                 assert not os.path.exists(os.path.join(folder, dist))
                 assert not os.path.exists(os.path.join(folder, dist + '.tar.bz2'))
@@ -1264,22 +1266,22 @@ def test(recipedir_or_package_or_metadata, config, move_broken=True):
 
     clean_pkg_cache(metadata.dist(), config)
 
+    test_package_name = (recipedir_or_package_or_metadata.dist()
+                         if hasattr(recipedir_or_package_or_metadata, 'dist')
+                         else recipedir_or_package_or_metadata)
+
     create_files(config.test_dir, metadata, config)
     # Make Perl or Python-specific test files
-    if metadata.name().startswith('perl-'):
-        pl_files = create_pl_files(config.test_dir, metadata)
-        py_files = False
-        lua_files = False
-    else:
-        py_files = create_py_files(config.test_dir, metadata)
-        pl_files = False
-        lua_files = False
+    pl_files = create_pl_files(config.test_dir, metadata)
+    py_files = create_py_files(config.test_dir, metadata)
+    r_files = create_r_files(config.test_dir, metadata)
+    lua_files = create_lua_files(config.test_dir, metadata)
     shell_files = create_shell_files(config.test_dir, metadata, config)
-    if not (py_files or shell_files or pl_files or lua_files):
-        print("Nothing to test for:", metadata.dist())
+    if not any([py_files, shell_files, pl_files, lua_files, r_files]):
+        print("Nothing to test for:", test_package_name)
         return True
 
-    print("TEST START:", metadata.dist())
+    print("TEST START:", test_package_name)
 
     if config.remove_work_dir:
         # Needs to come after create_files in case there's test/source_files
@@ -1307,12 +1309,14 @@ def test(recipedir_or_package_or_metadata, config, move_broken=True):
     if lua_files:
         # not sure how this shakes out
         specs += ['lua %s.*' % environ.get_lua_ver(config)]
+    if r_files:
+        # not sure how this shakes out
+        specs += ['r-base %s.*' % environ.get_r_ver(config)]
 
     create_env(config.test_prefix, specs, config=config)
 
     with utils.path_prepended(config.test_prefix):
-        env = dict(os.environ.copy())
-        env.update(environ.get_dict(config=config, m=metadata, prefix=config.test_prefix))
+        env = environ.get_dict(config=config, m=metadata, prefix=config.test_prefix)
         env["CONDA_BUILD_STATE"] = "TEST"
         if env_path_backup_var_exists:
             env["CONDA_PATH_BACKUP"] = os.environ["CONDA_PATH_BACKUP"]
@@ -1329,6 +1333,7 @@ def test(recipedir_or_package_or_metadata, config, move_broken=True):
 
     # Python 2 Windows requires that envs variables be string, not unicode
     env = {str(key): str(value) for key, value in env.items()}
+
     suffix = "bat" if utils.on_win else "sh"
     test_script = join(config.test_dir, "conda_test_runner.{suffix}".format(suffix=suffix))
 
@@ -1348,34 +1353,39 @@ def test(recipedir_or_package_or_metadata, config, move_broken=True):
             # use pythonw for import tests when osx_is_app is set
             if metadata.get_value('build/osx_is_app') and sys.platform == 'darwin':
                 test_python = test_python + 'w'
-            tf.write("{python} -s {test_file}\n".format(
+            tf.write('"{python}" -s "{test_file}"\n'.format(
                 python=config.test_python,
                 test_file=join(config.test_dir, 'run_test.py')))
             if utils.on_win:
                 tf.write("if errorlevel 1 exit 1\n")
         if pl_files:
-            tf.write("{perl} {test_file}\n".format(
-                perl=config.test_perl,
+            tf.write('"{perl}" "{test_file}"\n'.format(
+                perl=config.perl_bin(config.test_prefix),
                 test_file=join(config.test_dir, 'run_test.pl')))
             if utils.on_win:
                 tf.write("if errorlevel 1 exit 1\n")
         if lua_files:
-            tf.write("{lua} {test_file}\n".format(
-                lua=config.test_lua,
+            tf.write('"{lua}" "{test_file}"\n'.format(
+                lua=config.lua_bin(config.test_prefix),
                 test_file=join(config.test_dir, 'run_test.lua')))
             if utils.on_win:
                 tf.write("if errorlevel 1 exit 1\n")
+        if r_files:
+            tf.write('"{r}" "{test_file}"\n'.format(
+                r=config.r_bin(config.test_prefix),
+                test_file=join(config.test_dir, 'run_test.r')))
+            if utils.on_win:
+                tf.write("if errorlevel 1 exit 1\n")
         if shell_files:
             test_file = join(config.test_dir, 'run_test.' + suffix)
             if utils.on_win:
-                tf.write("call {test_file}\n".format(test_file=test_file))
+                tf.write('call "{test_file}"\n'.format(test_file=test_file))
                 if utils.on_win:
                     tf.write("if errorlevel 1 exit 1\n")
             else:
                 # TODO: Run the test/commands here instead of in run_test.py
-                tf.write("{shell_path} -x -e {test_file}\n".format(shell_path=shell_path,
-                                                                    test_file=test_file))
-
+                tf.write('"{shell_path}" -x -e "{test_file}"\n'.format(shell_path=shell_path,
+                                                                       test_file=test_file))
     if utils.on_win:
         cmd = ['cmd.exe', "/d", "/c", test_script]
     else:
@@ -1388,7 +1398,7 @@ def test(recipedir_or_package_or_metadata, config, move_broken=True):
     if need_cleanup:
         utils.rm_rf(recipe_dir)
 
-    print("TEST END:", metadata.dist())
+    print("TEST END:", test_package_name)
     return True
 
 
@@ -1491,7 +1501,7 @@ def build_tree(recipe_list, config, build_only=False, post=False, notest=False,
                             # IOError means recipe was not included with package. metadata instead
                             except IOError:
                                 test(metadata, config=config)
-                    built_packages.append(pkg)
+                        built_packages.append(pkg)
         except (NoPackagesFound, NoPackagesFoundError, Unsatisfiable, CondaError) as e:
             error_str = str(e)
             skip_names = ['python', 'r']
@@ -1524,7 +1534,7 @@ packages, the other package needs to be rebuilt
                         add_recipes.append(recipe_dir)
                 else:
                     raise RuntimeError("Can't build {0} due to unsatisfiable dependencies:\n{1}"
-                                       .format(recipe, e.pkgs) + "\n\n" + extra_help)
+                                       .format(recipe, error_str) + "\n\n" + extra_help)
             if retried_recipes.count(recipe) >= len(metadata.ms_depends('build')):
                 raise RuntimeError("Can't build {0} due to environment creation error:\n"
                                     .format(recipe) + error_str + "\n" + extra_help)
@@ -1616,12 +1626,18 @@ def handle_pypi_upload(wheels, config):
 
     wheels = utils.ensure_list(wheels)
 
-    for f in wheels:
-        try:
-            utils.check_call_env(args + [f])
-        except:
-            logging.getLogger(__name__).warn("wheel upload failed - is twine installed?"
-                                            "  Is this package registered?")
+    if config.anaconda_upload:
+        for f in wheels:
+            print("Uploading {}".format(f))
+            try:
+                utils.check_call_env(args + [f])
+            except:
+                logging.getLogger(__name__).warn("wheel upload failed - is twine installed?"
+                                                "  Is this package registered?")
+                logging.getLogger(__name__).warn("Wheel file left in {}".format(f))
+
+    else:
+        print("anaconda_upload is not set.  Not uploading wheels: {}".format(wheels))
 
 
 def print_build_intermediate_warning(config):
