@@ -180,6 +180,7 @@ def _update_paths(paths, mapping_dict):
             path['_path'] = mapping_dict[path['_path']]
     return updated_paths
 
+
 path_mapping_bat_proxy = (re.compile(r'bin\/(.+?)(\.py[c]?)?$'),
                            (r'Scripts/\1-script', r'Scripts/\1.bat'))
 
@@ -266,6 +267,11 @@ def get_pure_py_file_map(t, platform):
         fieldnames = ['prefix', 'type', 'path']
         csv_dialect = csv.Sniffer().sniff(has_prefix_files)
         csv_dialect.lineterminator = '\n'
+        for attr in ('delimiter', 'quotechar'):
+            if PY3 and hasattr(getattr(csv_dialect, attr), 'decode'):
+                setattr(csv_dialect, attr, getattr(csv_dialect, attr).decode())
+            elif not PY3 and hasattr(getattr(csv_dialect, attr), 'encode'):
+                setattr(csv_dialect, attr, getattr(csv_dialect, attr).encode())
         has_prefix_files = csv.DictReader(has_prefix_files.splitlines(), fieldnames=fieldnames,
                                           dialect=csv_dialect)
         # convenience: store list of dictionaries as map by path
