@@ -12,7 +12,7 @@ import subprocess
 
 # noqa here because PY3 is used only on windows, and trips up flake8 otherwise.
 from .conda_interface import text_type, PY3  # noqa
-from .conda_interface import root_dir, cc
+from .conda_interface import root_dir
 
 from conda_build.os_utils import external
 from conda_build import utils
@@ -25,13 +25,17 @@ def get_perl_ver(config):
 
 
 def get_lua_ver(config):
-    return str(config.CONDA_LUA)
+    return config.CONDA_LUA
 
 
 def get_py_ver(config):
     return '.'.join(str(config.CONDA_PY))
 
 
+def get_r_ver(config):
+    return config.CONDA_R
+
+
 def get_npy_ver(config):
     if config.CONDA_NPY:
         # Convert int -> string, e.g.
@@ -214,9 +218,9 @@ def get_dict(config, m=None, prefix=None):
     d = conda_build_vars(prefix, config)
 
     # languages
-    d.update(python_vars(config))
-    d.update(perl_vars(config))
-    d.update(lua_vars(config))
+    d.update(python_vars(config, prefix))
+    d.update(perl_vars(config, prefix))
+    d.update(lua_vars(config, prefix))
 
     if m:
         d.update(meta_vars(m, config))
@@ -250,12 +254,12 @@ def conda_build_vars(prefix, config):
     }
 
 
-def python_vars(config):
+def python_vars(config, prefix):
     d = {
-        'PYTHON': config.build_python,
+        'PYTHON': config.python_bin(prefix),
         'PY3K': str(config.PY3K),
-        'STDLIB_DIR': utils.get_stdlib_dir(config.build_prefix),
-        'SP_DIR': utils.get_site_packages(config.build_prefix),
+        'STDLIB_DIR': utils.get_stdlib_dir(prefix),
+        'SP_DIR': utils.get_site_packages(prefix),
         'PY_VER': get_py_ver(config),
         'CONDA_PY': str(config.CONDA_PY),
     }
@@ -267,14 +271,15 @@ def python_vars(config):
     return d
 
 
-def perl_vars(config):
+def perl_vars(config, prefix):
     return {
+        'PERL': config.perl_bin(prefix),
         'PERL_VER': get_perl_ver(config),
     }
 
 
-def lua_vars(config):
-    lua = config.build_lua
+def lua_vars(config, prefix):
+    lua = config.lua_bin(prefix)
     if lua:
         return {
             'LUA': lua,
@@ -504,10 +509,3 @@ class Environment(object):
             p, v, b = i['name'], i['version'], i['build']
             specs.append('{} {} {}'.format(p, v, b))
         return specs
-
-
-if __name__ == '__main__':
-    e = get_dict(cc)
-    for k in sorted(e):
-        assert isinstance(e[k], str), k
-        print('%s=%s' % (k, e[k]))
