@@ -36,6 +36,7 @@ if PY3:
     import urllib.request as urllib
     # NOQA because it is not used in this file.
     from contextlib import ExitStack  # NOQA
+    PermissionError = PermissionError
 else:
     import urlparse
     import urllib
@@ -249,6 +250,8 @@ def merge_tree(src, dst, symlinks=False, timeout=90, lock=None, locking=True):
 #    at any time, but the lock within this process should all be tied to the same tracking
 #    mechanism.
 _locations = {}
+_lock_folders = (os.path.join(root_dir, 'locks'),
+                 os.path.expanduser(os.path.join('~', '.conda_build_locks')))
 
 
 def get_lock(folder, timeout=90, filename=".conda_lock"):
@@ -263,15 +266,22 @@ def get_lock(folder, timeout=90, filename=".conda_lock"):
     lock_filename = base64.urlsafe_b64encode(b_location)[:20]
     if hasattr(lock_filename, 'decode'):
         lock_filename = lock_filename.decode()
-    locks_dir = os.path.join(root_dir, 'locks')
-    if not os.path.isdir(locks_dir):
-        os.makedirs(locks_dir)
-    lock_file = os.path.join(locks_dir, lock_filename)
-    if not os.path.isfile(lock_file):
-        with open(lock_file, 'a') as f:
-            f.write(location)
-    if location not in _locations:
-        _locations[location] = filelock.FileLock(lock_file, timeout)
+    for locks_dir in _lock_folders:
+        try:
+            if not os.path.isdir(locks_dir):
+                os.makedirs(locks_dir)
+            lock_file = os.path.join(locks_dir, lock_filename)
+            if not os.path.isfile(lock_file):
+                with open(lock_file, 'a') as f:
+                    f.write(location)
+            if location not in _locations:
+                _locations[location] = filelock.FileLock(lock_file, timeout)
+            break
+        except (OSError, IOError):
+            continue
+    else:
+        raise RuntimeError("Could not write locks folder to either system location ({0})"
+                           "or user location ({1}).  Aborting.".format(*_lock_folders))
     return _locations[location]
 
 
@@ -311,7 +321,10 @@ unxz is required to unarchive .xz source files.
         check_call_env([unxz, '-f', '-k', tarball])
         tarball = tarball[:-3]
     t = tarfile.open(tarball, mode)
-    t.extractall(path=dir_path)
+    if not PY3:
+        t.extractall(path=dir_path.encode(codec))
+    else:
+        t.extractall(path=dir_path)
     t.close()
 
 
@@ -533,6 +546,7 @@ def path_prepended(prefix):
     finally:
         os.environ['PATH'] = old_path
 
+
 bin_dirname = 'Scripts' if sys.platform == 'win32' else 'bin'
 
 entry_pat = re.compile('\s*([\w\-\.]+)\s*=\s*([\w.]+):([\w.]+)\s*$')
@@ -548,18 +562,23 @@ def iter_entry_points(items):
 
 def create_entry_point(path, module, func, config):
     pyscript = PY_TMPL % {'module': module, 'func': func}
-    if sys.platform == 'win32':
-        with open(path + '-script.py', 'w') as fo:
-            if os.path.isfile(os.path.join(config.build_prefix, 'python_d.exe')):
-                fo.write('#!python_d\n')
-            fo.write(pyscript)
-        copy_into(join(dirname(__file__), 'cli-{}.exe'.format(config.arch)),
-                  path + '.exe', config.timeout)
-    else:
+    if config.noarch:
         with open(path, 'w') as fo:
-            fo.write('#!%s\n' % config.build_python)
             fo.write(pyscript)
         os.chmod(path, 0o775)
+    else:
+        if sys.platform == 'win32':
+            with open(path + '-script.py', 'w') as fo:
+                if os.path.isfile(os.path.join(config.build_prefix, 'python_d.exe')):
+                    fo.write('#!python_d\n')
+                fo.write(pyscript)
+            copy_into(join(dirname(__file__), 'cli-{}.exe'.format(config.arch)),
+                    path + '.exe', config.timeout)
+        else:
+            with open(path, 'w') as fo:
+                fo.write('#!%s\n' % config.build_python)
+                fo.write(pyscript)
+            os.chmod(path, 0o775)
 
 
 def create_entry_points(items, config):
